#pragma once

#include "Owned_string.h"
#include <iostream>

// Structure for representing ranges of characters, usually for the
// purpose of comparing them. The idea is that we have constructors for
// a bunch of different ways that strings might be specified, and each
// gets converted to a string_view [begin, end).
struct string_view
{
    // Constructs a string_view from `begin` and `end` directly.
    string_view(const char* begin, const char* end);

    // Constructs a string_view from the start and the size.
    string_view(const char*, size_t);

    // Constructs a string_view from the contents of a `String`.
    string_view(const Owned_string*);

    // Constructs a string_view from the contents of a `String`.
    string_view(Owned_string const&);

    // Constructs a string_view from a '\0'-terminated C-style string.
    explicit string_view(const char*);

    // Constructs a string_view from a string literal using its static size.
    template <size_t N>
    string_view(const char (&s) [N]);

    size_t size() const;

    /*
     * Member variables
     */

    const char* begin;
    const char* end;
};

// Overloads == for `string_view`s.
bool operator==(string_view, string_view);

// Overloads != for `string_view`s.
bool operator!=(string_view, string_view);

// Overloads stream insertion (printing)
std::ostream& operator<<(std::ostream&, string_view);

// Templates need to be defined in the .h file, not the .cpp file.
// (We subtract 1 from N because N will include the string literal's
// '\0' terminator.)
template <size_t N>
string_view::string_view(const char (&s) [N])
        : string_view(s, N - 1)
{ }

